@@ -21,10 +21,9 @@ from shutil import copyfile
 from py4ami.util import Util
 from py4ami.constants import CEV_OPEN_DICT_DIR, OV21_DIR, DICT_AMI3
 from py4ami.ami_html import HtmlUtil, CSSStyle, H_A, H_SPAN, H_BODY, H_DIV, H_UL, H_LI, A_ID, \
-    A_HREF, A_NAME, A_TITLE
+    A_HREF, A_NAME, A_TITLE, A_TERM
 from py4ami.util import AbstractArgs
 from py4ami.wikimedia import WikidataSparql, WikidataLookup, WikidataPage
-from test.resources import Resources
 
 # elements in amidict
 DICTIONARY = "dictionary"
@@ -644,7 +643,6 @@ class AmiDictionary:
     def write_to_dir(self, directory):
         """write dictionary to file based on title
         :param directory: directory which will contain <title>.xml"""
-        from lxml import etree
         if not directory:
             print(f"None directory")
         if not directory.exists():
@@ -741,15 +739,25 @@ class AmiDictionary:
     def markup_html_from_dictionary(self, target_path, output_path, background_color):
         term_set = self.get_or_create_term_set()
         re_join = '|'.join(term_set)
-        rec = re.compile(f"(.*?)({re_join})(.*)")
+        try:
+            rec = re.compile(f"(.*?)({re_join})(.*)")
+        except Exception as e:
+            logging.error(f"dictionary {self.title}: Cannot parse terms into regex: \n{re_join}\n please remove or escape characters")
+            return None
         target_elem = lxml.etree.parse(str(target_path))
         div_spans = target_elem.xpath(f".//{H_DIV}/{H_SPAN}")
         for span in div_spans:
             text = span.text
             new_elems = HtmlUtil.split_span_at_match(span, rec, new_tags=[H_SPAN, H_A, H_SPAN],
-                                         recurse=True, id_root=f"{span.attrib['id']}_", id_counter=0)
+                                                     recurse=True, id_root=f"{span.attrib['id']}_", id_counter=0)
         self.convert_matched_spans_to_a(target_elem)
 
+        id_dict, multidict = self.write_annotated_html(background_color, output_path, target_elem)
+
+        self.write_index(id_dict, multidict, output_path)
+
+    def write_annotated_html(self, background_color, output_path,
+                             target_elem):
         a_elems = target_elem.xpath(f".//{H_A}")
         print(f" a_elems {len(a_elems)}")
         multidict = dict()
@@ -766,7 +774,7 @@ class AmiDictionary:
             following_text = following[0].text if len(following) > 0 else None
             id_dict[a_elem.attrib[A_ID]] = preceding_text, (a_elem.text), following_text
             # add to multidict
-            if not a_elem.text in multidict:
+            if a_elem.text not in multidict:
                 multidict[a_elem.text] = []
             multidict[a_elem.text].append(a_elem.attrib[A_ID])
             match_counter[a_elem.text] += 1
@@ -775,7 +783,12 @@ class AmiDictionary:
                 href = entry.attrib["wikidataID"]
                 if href:
                     a_elem.attrib[A_HREF] = WIKIDATA_SITE + href
-                    a_elem.attrib[A_TITLE] = entry.attrib[A_NAME]
+                name = entry.attrib.get(A_NAME)
+                if name:
+                    a_elem.attrib[A_TITLE] = name
+                else:
+                    a_elem.attrib[A_TITLE] = entry.attrib.get(A_TERM)
+
             else:
                 # print(f" {text} has no wikidataIO")
                 pass
@@ -784,6 +797,11 @@ class AmiDictionary:
         with open(str(output_path), "wb") as f:
             f.write(lxml.etree.tostring(target_elem))
             print(f"wrote {output_path}")
+
+        return id_dict, multidict
+
+    def write_index(self, id_dict, multidict, output_path):
+        output_path = Path(output_path)
         html = HtmlUtil.create_skeleton_html()
         body = html.xpath(f".//{H_BODY}")[0]
         ul = lxml.etree.SubElement(body, H_UL)
@@ -804,13 +822,12 @@ class AmiDictionary:
                 if prec_follow[2]:
                     span = lxml.etree.SubElement(li2, H_SPAN)
                     span.text = prec_follow[2]
-        list_path = Path(output_path.parent, "links.html")
+        logging.info(f"outpath {output_path} {output_path.parent}")
+        list_path = Path(output_path.parent, "index.html")
         with open(str(list_path), "wb") as f:
             f.write(lxml.etree.tostring(html))
             print(f"wrote {list_path}")
 
-
-
     def convert_matched_spans_to_a(self, chap_elem):
         """some matches are complete span and need converting to <a>
         """
@@ -958,7 +975,7 @@ class AmiDictionary:
             if child.tag in AmiDictionary.ALLOWED_CHILDREN:
                 pass
             else:
-                print(f"forbidden child {child.tag} in {AmiDictionary.ALLOWED_CHILDREN}")
+                print(f"forbidden child of {self.root.tag}: {child.tag} ; allowed = {AmiDictionary.ALLOWED_CHILDREN}")
 
 
 class AmiSynonym:
@@ -1129,6 +1146,7 @@ class AmiDictionaries:
 # this should not be here but I can't load it from an outside file
 XML_LANG = '{http://www.w3.org/XML/1998/namespace}lang'
 
+
 class AMIDictError(Exception):
     """Basic exception for errors raised in AMIDict"""
 
@@ -1156,17 +1174,11 @@ class AmiDictArgs(AbstractArgs):
         self.wikipedia = None
         self.ami_dict = None
 
-    @property
-    def module_stem(self):
-        """name of module"""
-        return Path(__file__).stem
-
-    def create_arg_parser(self):
-        """
-        creates adds the arguments for pyami commandline
-
-        """
-        self.parser = argparse.ArgumentParser(description='AMI dictionary creation, validation, editing')
+    def add_arguments(self):
+        if self.parser is None:
+            self.parser = argparse.ArgumentParser()
+        """adds arguments to a parser or subparser"""
+        self.parser.description = 'AMI dictionary creation, validation, editing'
         self.parser.add_argument(f"--{DELETE}", type=str, nargs="+",
                                  help="list of entries (terms) to delete ? duplicates (NYI)")
         self.parser.add_argument(f"--{DICT}", type=str, nargs=1,
@@ -1179,15 +1191,14 @@ class AmiDictArgs(AbstractArgs):
                                  help="replace any existing entries/attributes (default preserve) (NYI)")
         self.parser.add_argument(f"--{SYNONYM}", type=str, nargs="+",
                                  help="add sysnonyms (from Wikidata) for terms (NYI)")
-        self.parser.add_argument(f"--{VALIDATE}", action="store_true", help="validate dictionary (NYI)")
+        self.parser.add_argument(f"--{VALIDATE}", action="store_true", help="validate dictionary")
         self.parser.add_argument(f"--{WIKIDATA}", type=str, nargs="*", help="add WikidataIDs (NYI)")
         self.parser.add_argument(f"--{WIKIPEDIA}", type=str, nargs="*",
                                  help="add Wikipedia link/s (forces --{WIKIDATA}) (NYI)")
         self.parser.add_argument(f"--{WORDS}", type=str, nargs=1,
                                  help="path/file with words to make or edit dictionary")
-        # self.parser.add_argument(f"--{SORT}", type=str, nargs=1, help="sort by term, sort synonyms, sort by weight (NYI)")
-        # self.parser.add_argument(f"--{CATEGORY}", type=str, nargs=1, help="annotate by category (NYI)")
-        return self.parser
+        # parser.add_argument(f"--{SORT}", type=str, nargs=1, help="sort by term, sort synonyms, sort by weight (NYI)")
+        # parser.add_argument(f"--{CATEGORY}", type=str, nargs=1, help="annotate by category (NYI)")
 
     # class AmiDictArgs:
     def process_args(self):
@@ -1207,25 +1218,40 @@ class AmiDictArgs(AbstractArgs):
         self.parser.add_argument(f"--wikidata", type=str, nargs="*", help="add WikidataIDs")
         self.parser.add_argument(f"--wikipedia", type=str, nargs="*", help="add Wikipedia link/s")
         """
-        if self.arg_dict:
-            self.delete = self.arg_dict.get(DELETE)
-            self.dictfile = self.arg_dict.get(DICT)
-            self.filter = self.arg_dict.get(FILTER)
-            self.language = self.arg_dict.get(LANGUAGE)
-            self.metadata = self.arg_dict.get(METADATA)
-            self.replace = self.arg_dict.get(REPLACE)
-            self.synonym = self.arg_dict.get(SYNONYM)
-            self.validate = self.arg_dict.get(VALIDATE)
-            self.wikidata = self.arg_dict.get(WIKIDATA)
-            self.wikipedia = self.arg_dict.get(WIKIPEDIA)
-            self.words = self.arg_dict.get(WORDS)
+        if not self.arg_dict:
+            print(f"no arg_dict given, no actiom")
 
-            ami_dict = self.build_or_edit_dictionary()
-            if self.dictfile and ami_dict is not None:
+        self.delete = self.arg_dict.get(DELETE)
+        self.dictfile = self.arg_dict.get(DICT)
+        self.filter = self.arg_dict.get(FILTER)
+        self.language = self.arg_dict.get(LANGUAGE)
+        self.metadata = self.arg_dict.get(METADATA)
+        self.replace = self.arg_dict.get(REPLACE)
+        self.synonym = self.arg_dict.get(SYNONYM)
+        self.validate = self.arg_dict.get(VALIDATE)
+        self.wikidata = self.arg_dict.get(WIKIDATA)
+        self.wikipedia = self.arg_dict.get(WIKIPEDIA)
+        self.words = self.arg_dict.get(WORDS)
+
+        if self.dictfile:
+            if self.ami_dict is not None:
                 with open(self.dictfile, "w") as f:
                     self.ami_dict.write(self.dictfile)
                     print(f"wrote dict: {self.dictfile}")
+            else:
+                # print(f"reading dictionary {self.dictfile}")
+                self.ami_dict = self.build_or_edit_dictionary()
+                if self.ami_dict is None:
+                    print(f"failed to read/compile dictionaty {self.dictfile}")
 
+        if self.validate:
+            if self.ami_dict is None:
+                print(f"no dictionary givem")
+            else:
+                print(f"VALIDATING {self.ami_dict}")
+                status = self.validate_dict()
+
+        if self.wikidata:
             self.add_wikidata_to_dict()
             status = self.validate_dict()
 
@@ -1243,17 +1269,19 @@ class AmiDictArgs(AbstractArgs):
 
     def build_or_edit_dictionary(self):
         if not self.dictfile:
-            print("No dictionary given")
+            print("No dictionary file given")
             return None
+
         if not self.words:
-            print("No input words given")
-            return None
-        if not Path(self.words).exists():
-            raise FileNotFoundError(f"wordfile {self.words} does not exist.")
-        title = Path(self.words).stem
-        print(f"creating {self.dictfile} from {self.words}")
-        word_path = Path(self.words)
-        self.ami_dict, _ = AmiDictionary.create_dictionary_from_wordfile(wordfile=word_path, title=None, desc=None)
+            print(f"reading {self.dictfile} as dictionary")
+            self.ami_dict = AmiDictionary.create_from_xml_file(self.dictfile)
+        else:
+            if not Path(self.words).exists():
+                raise FileNotFoundError(f"wordfile {self.words} does not exist.")
+            title = Path(self.words).stem
+            print(f"creating {self.dictfile} from {self.words}")
+            word_path = Path(self.words)
+            self.ami_dict, _ = AmiDictionary.create_dictionary_from_wordfile(wordfile=word_path, title=None, desc=None)
         return self.ami_dict
 
     def add_wikidata_to_dict(self, description_regex=None):
@@ -1316,11 +1344,16 @@ class AmiDictArgs(AbstractArgs):
     def validate_dict(self):
         status = False
         if self.dictfile and Path(self.dictfile).exists():
-            print(f"validate {self.validate}")
+            self.ami_dict.check_validity()
         else:
             print(f"requires existing dictionary")
         return status
 
+    @property
+    def module_stem(self):
+        """name of module"""
+        return Path(__file__).stem
+
 
 class AmiDictValidator:
     def __init__(self, dictionary=None, path=None):
@@ -1371,20 +1404,20 @@ def main(argv=None):
         print(f"***Cannot run amidict***; see output for errors: {e} ")
 
 
-def test_process_args_build_dictionary():
-    words0_txt = f"{Path(Resources.TEST_RESOURCES_DIR, 'words0.txt')}"
-    print(f"words {words0_txt}")
-    with open(words0_txt, "r") as f:
-        print(f"words ==> {f.readlines()}")
-
-    words0_xml = f"{Path(Resources.TEMP_DIR, 'words0.xml')}"
-    print(f" s1 {sys.argv}")
-    # sys.argv = ['/Applications/PyCharm CE.app/Contents/plugins/python-ce/helpers/pycharm/_jb_pytest_runner.py', 'ami_dict.py::test_process_args', "--words", f"{words0_txt}", "--dict", f"{words0_xml}", "--validate", "--wikidata", "label"]
-    # print(f" s2 {sys.argv}")
-    argv = ["dummy_prog_name", "--words", f"{words0_txt}", "--dict", f"{words0_xml}", "--validate", "--wikidata",
-            "label"]
-    print(f" s2 {argv}")
-    main(argv=argv)
+# def test_process_args_build_dictionary():
+#     words0_txt = f"{Path(Resources.TEST_RESOURCES_DIR, 'words0.txt')}"
+#     print(f"words {words0_txt}")
+#     with open(words0_txt, "r") as f:
+#         print(f"words ==> {f.readlines()}")
+#
+#     words0_xml = f"{Path(Resources.TEMP_DIR, 'words0.xml')}"
+#     print(f" s1 {sys.argv}")
+#     # sys.argv = ['/Applications/PyCharm CE.app/Contents/plugins/python-ce/helpers/pycharm/_jb_pytest_runner.py', 'ami_dict.py::test_process_args', "--words", f"{words0_txt}", "--dict", f"{words0_xml}", "--validate", "--wikidata", "label"]
+#     # print(f" s2 {sys.argv}")
+#     argv = ["dummy_prog_name", "--words", f"{words0_txt}", "--dict", f"{words0_xml}", "--validate", "--wikidata",
+#             "label"]
+#     print(f" s2 {argv}")
+#     main(argv=argv)
 
 
 if __name__ == "__main__":
