@@ -13,23 +13,21 @@ import argparse
 from enum import Enum
 from abc import ABC, abstractmethod
 # local
+from py4ami.ami_dict import AmiDictionary, AmiDictArgs
 from py4ami.ami_convert import ConvType, Converters
 from py4ami.ami_sections import AMIAbsSection
-from py4ami.ami_dict import AmiDictionary
+from py4ami.ami_html import HTMLArgs
 from py4ami.examples import Examples
 from py4ami.file_lib import FileLib
-from py4ami.pdfreader import Svg2PageConverter, Page2SectConverter, Xml2HtmlConverter, Xml2TxtConverter, \
-    Pdf2SvgConverter
-from py4ami.ami_project import CProject, CTree, CSubDir
+from py4ami.pdfreader import Svg2PageConverter, Page2SectConverter, Xml2HtmlConverter, Xml2TxtConverter, Pdf2SvgConverter
+from py4ami.ami_pdf import PDFArgs
+from py4ami.ami_project import CProject, CTree, CSubDir, ProjectArgs
 from py4ami.symbol import SymbolIni
 from py4ami.text_lib import TextUtil, DSLParser
 from py4ami.util import AmiLogger, Util
 from py4ami.wikimedia import WikidataLookup
 from py4ami.xml_lib import XmlLib
 
-logging.debug("loading pyamix.py")
-logging.warning(Path(__file__))
-
 
 class PyAMI:
     """ main entry point for running pyami
@@ -83,6 +81,12 @@ class PyAMI:
     SPECIAL_SYMBOLS = ["_proj"]
     LOGLEVEL = "loglevel"
 
+# parsers
+    DICT_PARSER = "DICT"
+    HTML_PARSER = "HTML"
+    PDF_PARSER = "PDF"
+    PROJECT_PARSER = "PROJECT"
+
     logger = logging.getLogger("pyami")
     symbol_ini = None
 
@@ -152,11 +156,24 @@ class PyAMI:
     def create_arg_parser(self):
         """creates adds the arguments for pyami commandline
         """
+
+        def run_dict(self):
+            print(f"run dict pyamix")
+
+        def run_pdf(args):
+            print(f"run pdf")
+
+        def run_project():
+            print(f"run project {self.args}")
+
+        version = self.version()
         parser = argparse.ArgumentParser(
-            description='Search sections with dictionaries and patterns')
+            description='Search sections with dictionaries and patterns V0.0.25')
         # apply_choices = [self.PDF2TXT, self.PDF2SVG, self.SVG2XML, self.TXT2SENT, self.XML2HTML, self.XML2TXT]
         apply_choices = ConvType.list_values()
         self.logger.debug(f"ch {apply_choices}")
+        parser.add_argument('--version', action="store_true",
+                            help=f"show version {version}")
         parser.add_argument('--apply', nargs="+",
                             #                            choices=['pdf2txt', 'txt2sent', 'xml2txt'],
                             choices=apply_choices,
@@ -213,8 +230,19 @@ class PyAMI:
         #                     choices=TestFile.OPTIONS,  # tests and/or setup/teardown
         #                     help='run tests for modules; no selection runs all')
         # TODO should tests be run from this menu
+
+        subparsers = parser.add_subparsers(help='subcommands', dest="command")
+
+        dict_parser = AmiDictArgs().make_sub_parser(subparsers)
+        html_parser = HTMLArgs().make_sub_parser(subparsers)
+        pdf_parser = PDFArgs().make_sub_parser(subparsers)
+        project_parser = ProjectArgs().make_sub_parser(subparsers)
+
+        parser.epilog = "other entry points run as 'python -m py4ami.ami_dict args' also ami_pdf, ami_project"
+
         return parser
 
+
     def commandline(self, commandline: str) -> None:
         """runs a commandline as a single string
         """
@@ -248,6 +276,7 @@ class PyAMI:
 
         """
         if isinstance(args, str):
+            args =  args.strip()
             args = args.split(" ")
 
         self.logger.debug(f"********** raw arglist {args}")
@@ -308,31 +337,52 @@ class PyAMI:
         There will be more here
 
          """
+        # print(f"RUN ARGUMENTS on {self} {self.args}")
         # path workflow
         self.wikipedia_lookup = WikidataLookup()
-        self.logger.warning(f"commandline args {self.args}")
-        self.logger.warning(f"args: {self.args}")
+        self.logger.debug(f"commandline args {self.args}")
+        subparser_type = self.args.get("command")
+        print(f" COMMAND: {subparser_type} {self.args}")
 
+        # if "func" in self.args:
+        #     f_func = self.args["func"]
+        #     print(f"FUNC {f_func}")
+        #     aa = f_func()
+        #     print(f"aa {aa}")
+        # messy - we need to use polymorphism
+        if not subparser_type:
+            abstract_args = None
+        elif subparser_type == "DICT":
+            abstract_args = AmiDictArgs()
+        elif subparser_type == "HTML":
+            abstract_args = HTMLArgs()
+        elif subparser_type == "PDF":
+            abstract_args = PDFArgs()
+        elif subparser_type == "PROJECT":
+            abstract_args = ProjectArgs()
+
+        if abstract_args:
+            abstract_args.parse_and_process1(self.args)
+        else:
+            self.run_core_mathods()
+
+    def run_core_mathods(self):
+        print(f"run_core")
         if self.FLAGS in self.args and self.args[self.FLAGS] is not None:
             self.add_flags()
-
         if self.CONFIG in self.args and self.args[self.CONFIG] is not None:
             self.apply_config()
-
         if self.EXAMPLES in self.args:
             example_args = self.args[self.EXAMPLES]
             if example_args is not None:
                 self.logger.debug(f" examples args: {example_args}")
                 Examples(self).run_examples(example_args)
-
         if self.COPY in self.args and not self.args[self.COPY] is None:
             self.logger.warning(f"COPY {self.args[self.COPY]}")
             self.copy_files()
-
         if self.PROJ in self.args:
             if self.SECT in self.args or self.GLOB in self.args or self.SPLIT in self.args:
                 self.run_project_workflow()
-
         # elif TestFile.TEST in self.args:
         #     self.logger.warning(f"TEST in **args {self.args}")
         #     TestFile.run_arg_tests(self.args)
@@ -391,7 +441,8 @@ class PyAMI:
             #     new_val = old_val
             # new_items[key] = new_val
         else:
-            self.logger.error(f"{old_val} unknown arg type {type(old_val)}")
+            if type(old_val) is str:
+                self.logger.error(f"substitution: {old_val} unknown arg type {type(old_val)}")
             new_val = old_val
 
         # special symbols such as "_proj"
@@ -1058,6 +1109,9 @@ class PyAMI:
         except Exception:
             return False
 
+    def version(self):
+        version = "0.0.24"
+        return version
 
 
 class ContentStore:
@@ -1109,7 +1163,7 @@ def main():
     """ main entry point for cmdline
 
     """
-    print(f"PYAMI")
+    # print(f"PYAMI")
     run_dsl = False
     run_tests = False  # needs re-implementing
     run_commands = True
